<?php

namespace Drupal\revision_graph\Controller;

use Drupal\Core\Link;
use Drupal\node\Controller\NodeController;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Url;
use Drupal\node\NodeStorageInterface;
use Drupal\node\NodeTypeInterface;
use Drupal\node\NodeInterface;
use Drupal\revision_graph\RevisionGraphStorage;
use Symfony\Component\DependencyInjection\ContainerInterface;
/**
 * Returns responses for Node routes.
 */
class RevisionGraphNodeController extends NodeController {

  /**
   * {@inheritdoc}
   */
  public function revisionOverview(NodeInterface $node) {
    $commits = $this->revisionOverviewAsFakeCommits($node);
    return [
      '#markup' => '<!-- DOM element in which we\'ll mount our graph -->
  <div id="graph-container">',
     '#attached' =>[
       'library' => ['revision_graph/revision_graph'],
       'drupalSettings' => [
         'revision_graph' => [
           'commits' => $commits,
           'initialBranch' => !empty($commits[0]['branch']) ? $commits[0]['branch'] : '',
         ],
       ],
     ],
    ];

  }

  /**
   * Load revision graph data from storage.
   *
   * @param NodeInterface $node
   *  Node for which we need to load the revision graph.
   * @return array|array[]
   *  List of revisions.
   */
  private function loadGraphData(NodeInterface $node) {
    $connection = \Drupal::service('database');
    $result = $connection->query("SELECT * FROM {revision_graph} WHERE primary_id = :entity_id", [':entity_id' => $node->id()])->fetchAll();
    $graph = [
      'map' => [],
      'branches' => [],
    ];
    foreach ($result as $r) {
      $graph['map'][$r->version_id] = $r->branch;
      if (!isset($graph['branches'][$r->branch])) {
        $graph['branches'][$r->branch] = $r->parent_version_id;
      }
    }
    return $graph;
  }

  /**
   * Load revision data as git commits.
   *
   * @param NodeInterface $node
   *  Node for which we need to load the revision graph.
   * @return array
   *  Revision data as list of commits.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  private function revisionOverviewAsFakeCommits(NodeInterface $node) {
    $graph_data = $this->loadGraphData($node);
    $branches = $graph_data['branches'];
    $graph_data = $graph_data['map'];

    $commits = [];
    $langcode = $node->language()->getId();
    $node_storage = \Drupal::EntityTypeManager()->getStorage('node');                                                                       
    $default_revision = $node->getRevisionId();
    foreach ($this->getRevisionIds($node, $node_storage) as $vid) {
      $branches_to_be_spawned = [];
      foreach ($branches as $branch_name => $branch_spawn_vid) {
        if ($branch_spawn_vid == $vid) {
          $branches_to_be_spawned[] = $branch_name;
        }
      }
      $commit = [
        'author' => '',
        'subject' => '',
        'style' => ['dot' => [ 'font' => '8pt courier' ] ],
        'branch' => isset($graph_data[$vid]) ? $graph_data[$vid] : RevisionGraphStorage::UNTRACKED_BANCH_NAME,
        'spawn_branches' => $branches_to_be_spawned,
        'name' => $vid,
        'dotText' => $vid,
      ];
      /** @var \Drupal\node\NodeInterface $revision */
      $revision = $node_storage->loadRevision($vid);
      $commit['author'] = $revision->getRevisionUser()->getAccountName() . ' <' . $revision->getRevisionUser()->getEmail()  . '>';
      $date = $this->dateFormatter->format($revision->revision_timestamp->value, 'short');
      if ($vid != $node->getRevisionId()) {
        $link = new Url('entity.node.revision', ['node' => $node->id(), 'node_revision' => $vid]);
      }
      else {
        $link = $node->toLink($date);
      }
      $commit['date'] = $date;
      // Subject can be a good candidate for moderation state.
      $commit['subject'] = $date;
      $commit['body'] = strip_tags($revision->revision_log->value);
      $commit['link'] = $link->toString();
      if (empty($commit['subject'])) {
        $commit['subject'] = '';
      }

      if ($vid == $default_revision) {
        $commit['tag'] = 'current';
      }
      $commits[] = $commit;
    }
    // $commits[0]['tag'] = 'latest';
    return array_reverse($commits);
  }
}
